function [parentFitness offspringFitness] = generateOffspring(x0, stdev, numOffspring, display)

    % ----------------------------------------------------------------------
    % GA options

    % Start with default options
    options = gaoptimset;
    % The population size
    options = gaoptimset(options,'PopulationSize', numOffspring);
    % We copy the parent numOffspring times to form the initial population
    initialPopulation = (x0*ones(1,numOffspring))';
    options = gaoptimset(options,'InitialPopulation', initialPopulation);
    % Only run one generation
    options = gaoptimset(options,'Generations', 1);
    % Disable crossover
    options = gaoptimset(options,'CrossoverFraction', 0);
    % Set number of elites
    options = gaoptimset(options,'Elite', 0);
    % We don't need this, since we specify an initial population. But gatool
    % defines the standard deviation of the Gaussian mutation as sigma times
    % the length of this interval
    options = gaoptimset(options,'PopInitRange', [0; 1]);
    options = gaoptimset(options,'MutationFcn', {@mutationgaussian stdev 0});
    
    % Run GA
    nvars = length(x0);
    [X,FVAL,REASON,OUTPUT,POPULATION,SCORES] = ga(@fsphere,nvars,[],[],[],[],[],[],[],options);
    
    if (display)
        
        % Plot the parent and the offspring on the fitness landscape (only for 1D or 2D functions)
        if (nvars == 1)
            figure;
            subplot(2,1,1);
            plot(x0, fsphere(x0), '.b');
            lim = max(abs(xlim));

            subplot(2,1,2);
            %plot(state.Population, zeros(length(state.Population),1), '.b');
            plot(POPULATION, SCORES, '.b');
            lim = max([lim abs(xlim)]);

            plot1D(@fsphere, [-lim lim]);
            hold on;
            plot(POPULATION, SCORES, '.b');
            xlim([-lim lim]);
            ylabel Fitness
            title('Offspring')

            subplot(2,1,1);
            plot1D(@fsphere, [-lim lim]);
            hold on;
            plot(x0, fsphere(x0), '.b');
            xlim([-lim lim]);
            ylabel Fitness
            title('Parent');

        elseif (nvars == 2)
            figure;
            subplot(2,1,1);
            plot(x0(1), x0(2), '.b');
            lim = max([abs(xlim) abs(ylim)]);

            subplot(2,1,2);
            plot(POPULATION(:,1), POPULATION(:,2), '.b');
            lim = max([lim abs(xlim) abs(ylim)]);

            plot2D(@fsphere, [-lim lim]);
            hold on;
            plot(POPULATION(:,1), POPULATION(:,2), '.b');
            xlim([-lim lim]);
            ylim([-lim lim]);
            axis square;
            box on;
            title('Offspring')
            xlabel x0;
            ylabel x1;
            zlabel Fitness;

            subplot(2,1,1);
            plot2D(@fsphere, [-lim lim]);
            hold on;
            plot(x0(1), x0(2), '.b');
            xlim([-lim lim]);
            ylim([-lim lim]);
            axis square;
            box on;
            title('Parent');
            h = colorbar('peer',gca,[0.7128 0.1084 0.03918 0.3436]);
            set(get(h,'ylabel'),'String','Fitness');
        end
    end
    
    parentFitness = fsphere(x0);
    offspringFitness = SCORES;

end


