function [bestIndividual bestFitness] = runGA(fitnessFunct, nvars, stdev, crossover, tournamentSize, popSize, popInitRange, numGenerations, display)

    % ----------------------------------------------------------------------
    % GA options
    
    % Start with default options
    options = gaoptimset;
    % The population size
    options = gaoptimset(options,'PopulationSize', popSize);
    % Unlimited number of generations
    options = gaoptimset(options,'Generations', numGenerations);
    % Disable detection of stalling
    options = gaoptimset(options,'StallGenLimit' ,Inf);
    options = gaoptimset(options,'StallTimeLimit' ,Inf);
    % Disable crossover
    options = gaoptimset(options,'CrossoverFraction', crossover);
    % Set number of elites
    options = gaoptimset(options,'Elite', 1);
    % Use our own tournament selection function (the one from the toolbox has a bug!)
    options = gaoptimset(options,'SelectionFcn' ,{  @tournamentselection tournamentSize  });
    % We don't need this, since we specify an initial population. But gatool
    % defines the standard deviation of the Gaussian mutation as sigma times
    % the length of this interval
    options = gaoptimset(options,'PopInitRange', popInitRange);
    % gatool defines the standard deviation of the Gaussian mutation as sigma times
    % the length of the popInitRange, but we want it to be the sigma that
    % we specify. The second argument may be used to shrink the mutation
    % rate (set to 1 to shrink linearly to 0 in the last generation)
    stdev = stdev / (popInitRange(2)-popInitRange(1));
    options = gaoptimset(options,'MutationFcn', {@mutationgaussian stdev 0});
    % This function is called at each generation---actually we don't use it
    % to do a plot, but to record the initial population in a global variable
    options = gaoptimset(options,'PlotFcns', {@myPlotFunction});
    
    
    % Run GA
    [X,FVAL,REASON,OUTPUT,POPULATION,SCORES] = ga(fitnessFunct,nvars,[],[],[],[],[],[],[],options);
    
    %global bestFitnesses;
    global initialPopulation;
    global initialScores;
        
    if (display)

        % Plot the parent and the offspring on the fitness landscape (only for 1D or 2D functions)
        if (nvars == 1)
            figure;
            subplot(2,1,1);
            plot(initialPopulation, initialScores, '.b');
            lim = max(abs(xlim));

            subplot(2,1,2);
            %plot(state.Population, zeros(length(state.Population),1), '.b');
            plot(POPULATION, SCORES, '.b');
            lim = max([lim abs(xlim)]);

            plot1D(fitnessFunct, [-lim lim]);
            hold on;
            plot(POPULATION, SCORES, '.b');
            xlim([-lim lim]);
            ylabel Fitness
            title('Final Population')

            subplot(2,1,1);
            plot1D(fitnessFunct, [-lim lim]);
            hold on;
            plot(initialPopulation, initialScores, '.b');
            xlim([-lim lim]);
            ylabel Fitness
            title('Initial Population');

        elseif (nvars == 2)
            figure;
            subplot(2,1,1);
            plot(initialPopulation(:,1), initialPopulation(:,2), '.b');
            lim = max([abs(xlim) abs(ylim)]);

            subplot(2,1,2);
            plot(POPULATION(:,1), POPULATION(:,2), '.b');
            lim = max([lim abs(xlim) abs(ylim)]);

            plot2D(fitnessFunct, [-lim lim]);
            hold on;
            plot(POPULATION(:,1), POPULATION(:,2), '.b');
            xlim([-lim lim]);
            ylim([-lim lim]);
            axis square;
            box on;
            title('Final population')
            xlabel x0;
            ylabel x1;
            zlabel Fitness;

            subplot(2,1,1);
            plot2D(fitnessFunct, [-lim lim]);
            hold on;
            plot(initialPopulation(:,1), initialPopulation(:,2), '.b');
            xlim([-lim lim]);
            ylim([-lim lim]);
            axis square;
            box on;
            title('Initial population');
            xlabel x0;
            ylabel x1;
            h = colorbar('peer',gca,[0.7128 0.1084 0.03918 0.3436]);
            set(get(h,'ylabel'),'String','Fitness');
        end
    end
    
    bestIndividual = X;
    bestFitness = FVAL;
    %bestFitnessOfEachGeneration = bestFitnesses;

end


